/**
 *	BOUND
 * ------------------------
 *   DISPLAY.C   |
 *   ------------
 *    Functions related to displaying stuff
**/


/////////////////////////////////////////////////////////////////
///////////////////////// INCLUDES //////////////////////////////
/////////////////////////////////////////////////////////////////

#include <psppower.h>
#include "graphics.h"
#include "fontloader.h"
#include "leveldata.h"
#include "input.h"
#include "common.c"


/////////////////////////////////////////////////////////////////
///////////////////////// FUNCTIONS /////////////////////////////
/////////////////////////////////////////////////////////////////

/* Flip routine */
void flip(void)
{
	flipScreen();
	sceDisplayWaitVblankStart();
}

/* Center text */
void centerText(int centerX, int centerY, char * centerText, int centerLength, u32 color)
{
	if (strlen(centerText) <= centerLength) {
		int center = ((centerX)-((strlen(centerText)/2)*8));
		Font_Print_Screen(myFont, center, centerY+11, centerText, color);
	}
	else {
		int center = ((centerX)-(centerLength/2)*8);
		char str[255];
		strncpy(str, centerText, centerLength);
		str[centerLength-3] = '.';
		str[centerLength-2] = '.';
		str[centerLength-1] = '.';
		str[centerLength] = '\0';
		Font_Print_Screen(myFont, center, centerY+11, str, color);
	}
}

/* Initiate the Images */
void initImages(void)
{
	// Images 
	player = loadImage("gfx/player.png");
	cursor = loadImage("gfx/cursor.png");

	arrow = loadImage("gfx/arrow.png");

	youDied = loadImage("gfx/youdied.png");
	deadTop = loadImage("gfx/deadTop.png");
	deadBot = loadImage("gfx/deadBot.png");
	
	enemy1 = loadImage("gfx/enemy1.png");
	enemy2 = loadImage("gfx/enemy2.png");
	enemy3 = loadImage("gfx/enemy3.png");
	enemy4 = loadImage("gfx/enemy4.png");

	stage1 = loadImage("gfx/levels/stage1.png");
	stage2 = loadImage("gfx/levels/stage2.png");
	stage3 = loadImage("gfx/levels/stage3.png");
	stage4 = loadImage("gfx/levels/stage4.png");

	door = loadImage("gfx/door.png");
	door2 = loadImage("gfx/door2.png");
	door3 = loadImage("gfx/door3.png");
	
	armLeft = loadImage("gfx/armleft.png");
	armRight = loadImage("gfx/armRight.png");

	menuBG = loadImage("gfx/menuBG.png");

	// Font
	myFont = Font_Load(FONT_PATH);
	Font_SetSize(myFont, 11);
}

/* Draw */
void draw(int curState)
{
	// Menu drawing
	if (curState == STATE_MENU) {
		//fillScreenRect(RGB(0, 0, 0), 0, 0, 480, 272);
		blitAlphaImageToScreen(0, 0, 480, 272, menuBG, 0, 0);
		Font_Print_Screen(myFont, 5, 15, "V1a", RGB(255, 255, 255));

		if (!(curMenu == 1)) centerText(480/2+5, 50, "Start Game", 10, RGB(255, 255, 255));
		if (!(curMenu == 2))centerText(480/2, 70, "Options", 7, RGB(255, 255, 255));
		if (!(curMenu == 3))centerText(480/2, 90, "Exit", 4, RGB(255, 255, 255));
		if (curMenu == 1) centerText(480/2+5, 50, "Start Game", 10, RGB(0, 0, 200));
		else if (curMenu == 2) centerText(480/2, 70, "Options", 7, RGB(0, 0, 200));
		else if (curMenu == 3) centerText(480/2, 90, "Exit", 4, RGB(0, 0, 200));
	}
	// Game drawing
	else if (curState == STATE_GAME) {
		// Clearscreen
		clearScreen(0);
		
		// Tut stage
		if (stage == 0) {
			if (tutState == 0) {
				// The level
				blitAlphaImageToScreen(0, 0, 480, 272, stage1, cameraX+stageX, cameraY+stageY);
				// The player
				blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
				// The instructs
				centerText(cameraX+480/2+30, cameraY+50, "This is you.", 12, RGB(255, 255, 255));
				centerText(cameraX+480/2+50, cameraY+60, "You're that little ball on the green platform.", 46, RGB(255, 255, 255));
				// The arrow
				blitAlphaImageToScreen(0, 0, 10, 25, arrow, cameraX+245, cameraY+100);
			}
			else if (tutState == 1) {
				// The level
				blitAlphaImageToScreen(0, 0, 480, 272, stage1, cameraX+stageX, cameraY+stageY);
				// The player
				blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
				// The instructs
				centerText(cameraX+480/2+30, cameraY+30, "This is your cursor.", 20, RGB(255, 255, 255));
				centerText(cameraX+480/2+50, cameraY+40, "The cursor is meant to maneuver you through the level.", 54, RGB(255, 255, 255));
				centerText(cameraX+480/2+50, cameraY+50, "Just press CROSS, and you'll go to that point.", 46, RGB(255, 255, 255));
				centerText(cameraX+480/2+50, cameraY+70, "If you press the cursor against the screen,", 43, RGB(255, 255, 255));
				centerText(cameraX+480/2+50, cameraY+80, "the screen will move with it.", 29, RGB(255, 255, 255));
				centerText(cameraX+480/2+50, cameraY+100, "Just make sure the screen doesn't go off of you.", 48, RGB(255, 255, 255));
				// The arrow
				blitAlphaImageToScreen(0, 0, 10, 25, arrow, cameraX+245/2+3, cameraY+114);
				// The cursor
				blitAlphaImageToScreen(0, 0, 10, 10, cursor, mouseX, mouseY);
			}
			else if (tutState == 2) {
				// The level
				cameraX = -235;
				cameraY = -60;
				blitAlphaImageToScreen(0, 0, 480, 272, stage1, cameraX+stageX, cameraY+stageY);
				// The player
				blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
				// The enemies
				for (i=1; i<=50; i++) {
					if (enemy[i].active)
						if (enemy[i].type == 1)
							blitAlphaImageToScreen(0, 0, 20, 20, enemy1, cameraX+enemy[i].x, cameraY+enemy[i].y);
				}
				// The instructs
				centerText(480/2+50, 30, "Your goal is to get through all of the obstacles.", 49, RGB(255, 255, 255));
				centerText(480/2+50, 40, "For every level, your goal is to reach the gray platform.", 57, RGB(255, 255, 255));
			}
			else if (tutState == 3) {
				centerText(480/2, 272/2, "Good Luck. :D", 13, RGB(255, 255, 255));
			}
			centerText(480/2+30, 250, "Press X to Continue", 19, RGB(255, 255, 255));
		}
		// Level 1
		else if (stage == 1) {
			// The level
			blitAlphaImageToScreen(0, 0, 480, 272, stage1, cameraX+stageX, cameraY+stageY);
			// Extra Check (before player)
			if (stageY+playerY > -cameraY) {
				if (touchColour(RGB(0, 0, 0)))
						dead = 1;
			}
			else {
				dead = 1;
			}
			// The player
			blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
			// The enemies
			for (i=1; i<=50; i++) {
				if (enemy[i].active)
					if (enemy[i].type == 1)
						blitAlphaImageToScreen(0, 0, 20, 20, enemy1, cameraX+enemy[i].x, cameraY+enemy[i].y);
			}
			// The cursor
			blitAlphaImageToScreen(0, 0, 10, 10, cursor, mouseX, mouseY);
		}
		// Level 2
		else if (stage == 2) {
			// The level
			blitAlphaImageToScreen(0, 0, 480, 272, stage2, cameraX+stageX, cameraY+stageY);
			// Extra Check (before player)
			if (stageY+playerY > -cameraY) {
				if (touchColour(RGB(0, 0, 0)))
						dead = 1;
			}
			else {
				dead = 1;
			}
			// The player
			blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
			// The enemies
			for (i=1; i<=50; i++) {
				if (enemy[i].active) {
					if (enemy[i].type == 1)
						blitAlphaImageToScreen(0, 0, 20, 20, enemy1, cameraX+enemy[i].x, cameraY+enemy[i].y);
					else if (enemy[i].type == 2)
						blitAlphaImageToScreen(0, 0, 30, 18, enemy2, cameraX+enemy[i].x, cameraY+enemy[i].y);
				}
			}
			// The cursor
			blitAlphaImageToScreen(0, 0, 10, 10, cursor, mouseX, mouseY);
		}
		// Level 3
		else if (stage == 3) {
			// The level
			blitAlphaImageToScreen(0, 0, 480, 272, stage3, cameraX+stageX, cameraY+stageY);
			// Extra Check (before player)
			if (stageY+playerY > -cameraY) {
				if (touchColour(RGB(0, 0, 0)))
						dead = 1;
			}
			else {
				dead = 1;
			}
			// The door
			if (doorSwitch)
				blitAlphaImageToScreen(0, 0, 14, 27, door, cameraX+stageX+292, cameraY+stageY+195);
			// The player
			blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
			// The enemies
			for (i=1; i<=50; i++) {
				if (enemy[i].active) {
					if (enemy[i].type == 1)
						blitAlphaImageToScreen(0, 0, 20, 20, enemy1, cameraX+enemy[i].x, cameraY+enemy[i].y);
					else if (enemy[i].type == 3)
						blitAlphaImageToScreen(0, 0, 24, 31, enemy3, cameraX+enemy[i].x, cameraY+enemy[i].y);
				}
			}
			// The cursor
			blitAlphaImageToScreen(0, 0, 10, 10, cursor, mouseX, mouseY);
		}
		// Level 4
		else if (stage == 4) {
			// The level
			blitAlphaImageToScreen(0, 0, 480, 272, stage4, cameraX+stageX, cameraY+stageY);
			// Extra Check (before player)
			if (stageY+playerY > -cameraY) {
				if (touchColour(RGB(0, 0, 0)))
						dead = 1;
			}
			else {
				dead = 1;
			}
			// The doors
			if (doorSwitch) {
				blitAlphaImageToScreen(0, 0, 59, 7, door2, cameraX+stageX+200, cameraY+stageY+226);
				blitAlphaImageToScreen(0, 0, 24, 23, door3, cameraX+stageX+392, cameraY+stageY+24);
			}
			// The boss
			if (enemy[1].active) {
				// body
				blitAlphaImageToScreen(0, 0, 40, 60, enemy4, cameraX+enemy[1].x, cameraY+enemy[1].y);
				// left arm
				blitAlphaImageToScreen(0, 0, 30, 10, armLeft, cameraX+enemy[1].x-30-armXLeft, cameraY+enemy[1].y+20+armYLeft);
				// right arm
				blitAlphaImageToScreen(0, 0, 30, 10, armRight, cameraX+enemy[1].x+40+armXRight, cameraY+enemy[1].y+20+armYRight);
			}
			// The player
			blitAlphaImageToScreen(0, 0, 10, 10, player, cameraX+stageX+playerX, cameraY+stageY+playerY);
			// The cursor
			blitAlphaImageToScreen(0, 0, 10, 10, cursor, mouseX, mouseY);

			// Countdown
			sprintf(countText, "Countdown: %d", countDown);
			//Font_Print_Screen(myFont, 3, 2+11+10, countText, RGB(220, 220, 220));
			printTextScreen(3, 2+11+6, countText, RGB(220, 220, 220));
		}
		// Level 5
		else if (stage >= 5) {
			// Message
			centerText(480/2+10, 272/2, "You Beat V1a of PSP Bound!", 26, RGB(255, 255, 255));
			centerText(480/2+10, 200, "Press X to Continue", 19, RGB(255, 255, 255));
			centerText(480/2, 240, "Email slasherzor@gmail.com to submit levels", 43, RGB(255, 255, 255));
			centerText(480/2+50, 250, "Submit levels with detail so I know where to put enemies, etc", 61, RGB(255, 255, 255));
			centerText(480/2+50, 260, "Submit your level with a preferred difficulty rating, 1-10", 58, RGB(255, 255, 255));
		}
		// etc

		if (dead) {
			blitAlphaImageToScreen(0, 0, 480, 272/2, deadTop, deadXtop, deadYtop);
			blitAlphaImageToScreen(0, 0, 480, 272/2, deadBot, deadXbot, deadYbot);
			blitAlphaImageToScreen(0, 0, 480, 272, youDied, deadX, deadY);
			centerText(deadXbot+480/2+40, deadYbot+5, "Please Press Start To Continue", 30, 0);
		}

		// HUD
		sprintf(HUD.holdLevel, "Level: %d", HUD.level);
		Font_Print_Screen(myFont, 3, 2+11, HUD.holdLevel, RGB(220, 220, 220));
	}
	else if (curState == STATE_OPTIONS) {
		//fillScreenRect(RGB(0, 0, 0), 0, 0, 480, 272);
		blitAlphaImageToScreen(0, 0, 480, 272, menuBG, 0, 0);
		Font_Print_Screen(myFont, 5, 15, "V1a", RGB(255, 255, 255));

		sprintf(cpuSpeed, "CPU Speed: %d", scePowerGetCpuClockFrequencyInt());
		sprintf(difficulty, "Difficulty: Easy");
		sprintf(sensitivity, "Sensitivity: %d", sens);
		if (!(menuOption == 1)) Font_Print_Screen(myFont, 50, 50, cpuSpeed, RGB(255, 255, 255));
		if (!(menuOption == 2)) Font_Print_Screen(myFont, 50, 65, difficulty, RGB(255, 255, 255));
		if (!(menuOption == 3)) Font_Print_Screen(myFont, 50, 80, sensitivity, RGB(255, 255, 255));
		if (!(menuOption == 4)) Font_Print_Screen(myFont, 50, 100, "Back", RGB(255, 255, 255));
		if (menuOption == 1) {
			Font_Print_Screen(myFont, 50, 50, cpuSpeed, RGB(0, 0, 200));
			centerText(480/2+30, 202, "Set the CPU speed. The higher, the faster", 42, RGB(255, 255, 255));
		}
		else if (menuOption == 2) {
			Font_Print_Screen(myFont, 50, 65, difficulty, RGB(0, 0, 200));
			centerText(480/2, 202, "Set the difficulty", 19, RGB(255, 255, 255));
		}
		else if (menuOption == 3) {
			Font_Print_Screen(myFont, 50, 80, sensitivity, RGB(0, 0, 200));
			centerText(480/2, 202, "Higher - Slower, Lower - Faster", 31, RGB(255, 255, 255));
		}
		else if (menuOption == 4) {
			Font_Print_Screen(myFont, 50, 100, "Back", RGB(0, 0, 200));
		}
	}
}


