/**
 *  BOUND
 * ------------------------
 *   INPUT.C   |
 *   ------------
 *    Functions used to control Bound
**/


/////////////////////////////////////////////////////////////////
////////////////////////// INCLUDES /////////////////////////////
/////////////////////////////////////////////////////////////////

#include <psppower.h>
#include "graphics.h"
#include "fontloader.h"
#include "display.h"
#include "leveldata.h"
#include "mp3player.h"
#include "common.c"



/////////////////////////////////////////////////////////////////
////////////////////////// GLOBALS //////////////////////////////
/////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////
//////////////////////// PROTOTYPES /////////////////////////////
/////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////
///////////////////////// FUNCTIONS /////////////////////////////
/////////////////////////////////////////////////////////////////

/* Detection code */
int touchColour(u32 color)
{
	if ((getPixelScreen(cameraX+stageX+playerX, cameraY+stageY+playerY) == color)				// Top left
		|| (getPixelScreen(cameraX+stageX+playerX+8, cameraY+stageY+playerY) == color)			// Top right
		|| (getPixelScreen(cameraX+stageX+playerX, cameraY+stageY+playerY+8) == color)			// Bot left
		|| (getPixelScreen(cameraX+stageX+playerX+9, cameraY+stageY+playerY+9) == color)		// Bot right
		|| (getPixelScreen(cameraX+stageX+playerX+3.5, cameraY+stageY+playerY) == color)		// Top mid
		|| (getPixelScreen(cameraX+stageX+playerX+9, cameraY+stageY+playerY+3.5) == color)		// Mid right
		|| (getPixelScreen(cameraX+stageX+playerX+3.5, cameraY+stageY+playerY+9) == color)		// Bot mid
		|| (getPixelScreen(cameraX+stageX+playerX, cameraY+stageY+playerY+3.5) == color))		// Left mid
		return 1;
	else
		return 0;
}

/* The mouse */
void mouse(int sensit)
{
	mouseMoveX = 0;
	mouseMoveY = 0;
	if (pad.Lx > 160 || pad.Lx < 96)
		mouseMoveX = (pad.Lx-128) / sensit;
				
	if (pad.Ly > 160 || pad.Ly < 96)
		mouseMoveY = (pad.Ly-128) / sensit;

	mouseX += mouseMoveX;
	mouseY += mouseMoveY;
	if (mouseX < 0) {
		mouseX = 0;
		cameraX -= mouseMoveX;
	}
	if (mouseX > 480) {
		mouseX = 480;
		cameraX -= mouseMoveX;
	}
	if (mouseY < 0) {
		mouseY = 0;
		cameraY -= mouseMoveY;
	}
	if (mouseY > 272) {
		mouseY = 272;
		cameraY -= mouseMoveY;
	}
}

/* Constantly check if you're dead yet */
void checkDead(void)
{
	if (dead) {
		if (deadY < 0) deadY += 5;
		if (deadYtop < 0) deadYtop += 4;
		if (deadYbot > 272/2) deadYbot -= 4;
	}
}

/* Initiate the Variables */
void initVars(void)
{
	// Regular Vars
	state = STATE_MENU;
	curMenu = STATE_MENU;
	HUD.level = 0;
	menuOption = 1;
}

/* Get random number */
int getRandomNum(int min, int max) {
	int diff = max - min;
	int res = (rand() % diff)+min;
	int i;
	// return a solid number
	for (i=min; i<=max; i++) {
		if ((res >= i-1) && (res < i))
			res = i-1;
	}
	return res;
}

/* Input (controls) */
void input(int curState)
{
	// Menu controls
	if (curState == STATE_MENU) {
		// Get control
		oldpad = pad;
		sceCtrlReadBufferPositive(&pad, 1);

		if (pad.Buttons != oldpad.Buttons) {
			// Up routine
			if ((pad.Buttons & UP) && (!(oldpad.Buttons & UP))) {
				if (curMenu > 1) curMenu--;
				else curMenu = 3;
				playSound(CLICK_PATH);
			}
			// Down routine
			else if ((pad.Buttons & DOWN) && (!(oldpad.Buttons & DOWN))) {
				if (curMenu < 3) curMenu++;
				else curMenu = 1;
				playSound(CLICK_PATH);
			}

			// Cross routine
			if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
				// Play click sound
				playSound(CLICK_PATH);
				if (curMenu == 1) {
					// load level
					read_config(CONFIG_PATH, &config);
					curMenuLevelPick = selectLevelScreen();
					if (curMenuLevelPick <= config.level) {
						loadLevel(curMenuLevelPick);
						state = STATE_GAME;
					}
					else {
						state = STATE_MENU;
					}
				}
				else if (curMenu == 2) state = STATE_OPTIONS;
				else if (curMenu == 3) state = STATE_EXIT;
			}
		}
			
	}
	// Game controls
	else if (curState == STATE_GAME) {
		// Get control
		sceCtrlSetSamplingCycle(0);
		sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
		oldpad = pad;
		sceCtrlReadBufferPositive(&pad, 1);

		// If not dead, continue
		if (!(dead)) {
			// Tut Stage
			if (stage == 0) {
				if (pad.Buttons != oldpad.Buttons) {
					if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
						playSound(NEXT_PATH);
						tutState++;
						if (tutState > 3) transitionLevel();
					}
				}
				if (tutState > 1) {
					// all enemies movements are the same, so a 'for' loop seems appropriate
					for (i=1; i<=50; i++) {
						if (enemy[i].active) {
							if (enemy[i].type == 1) {
								if (enemy[i].switcher) {
									if (enemy[i].y <= stageY+51)
										enemy[i].y++;
									else
										enemy[i].switcher = 0;
								}
								else if (!(enemy[i].switcher)) {
									if (enemy[i].y >= stageY)
										enemy[i].y--;
									else
										enemy[i].switcher = 1;
								}
							}
						}
					}
				}
			}
			// Level 1
			else if (stage == 1) {
				// Cross routine
				if (pad.Buttons != oldpad.Buttons) {
					if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
						gotoX = (mouseX-stageX)-cameraX;
						gotoY = (mouseY-stageY)-cameraY;
					}
				}

				// If getting colour from off the screen = freeze. This is meant to prevent that.
				if (stageY+playerY > -cameraY) {
					// Colour specifications
					// black areas
					if (touchColour(RGB(0, 0, 0)))
						dead = 1;
					// blue areas
					if (touchColour(RGB(0, 0, 255)))
						dead = 1;
					// finish area
					if (touchColour(RGB(192, 192, 192)))
						transitionLevel();
				}
				else {
					dead = 1;
				}

				// Stage 1 specific enemy movements
				// all enemies movements are the same, so a 'for' loop seems appropriate
				for (i=1; i<=50; i++) {
					if (enemy[i].active) {
						if (enemy[i].type == 1) {
							if (enemy[i].switcher) {
								if (enemy[i].y <= stageY+51)
									enemy[i].y++;
								else
									enemy[i].switcher = 0;
							}
							else if (!(enemy[i].switcher)) {
								if (enemy[i].y >= stageY)
									enemy[i].y--;
								else
									enemy[i].switcher = 1;
							}
						}
					}
				}
			}
			// Level 2
			else if (stage == 2) {
				// Cross routine
				if (pad.Buttons != oldpad.Buttons) {
					if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
						gotoX = (mouseX-stageX)-cameraX;
						gotoY = (mouseY-stageY)-cameraY;
					}
				}
				// If getting colour from off the screen = freeze. This is meant to prevent that.
				if (stageY+playerY > -cameraY) {
					// Colour specifications
					// black areas
					if (touchColour(RGB(0, 0, 0)))
						dead = 1;
					// red areas
					if (touchColour(RGB(200, 0, 0))) 
						dead = 1;
					// finish area
					if (touchColour(RGB(192, 192, 192)))
						transitionLevel();
				}
				else {
					dead = 1;
				}
				// enemies
				for (i=1; i<=50; i++) {
					if (enemy[i].active) {
						if (enemy[i].type == 1) {
							if (enemy[i].switcher == 1) {
								if (enemy[i].y < stageY+111) enemy[i].y++;
								else enemy[i].switcher = 2;
							}
							if (enemy[i].switcher == 2) {
								if (enemy[i].x < stageX+209) enemy[i].x++;
								else enemy[i].switcher = 3;
							}
							if (enemy[i].switcher == 3) {
								if (enemy[i].y > stageY+39) enemy[i].y--;
								else enemy[i].switcher = 4;
							}
							if (enemy[i].switcher == 4) {
								if (enemy[i].x > stageX+21) enemy[i].x--;
								else enemy[i].switcher = 1;
							}
						}
					}
					else if (enemy[i].type == 2) {
						enemyTimer--;
						if (enemyTimer < 0) {
							if ((enemyCur > 5) && (enemyCur < 15)) {
								enemy[enemyCur-1].active = 0;
								enemy[enemyCur].active = 1;
							}
							else {
								enemyCur = 5;
								enemy[14].active = 0;
								enemy[enemyCur].active = 1;
							}
							enemyCur++;
							enemyTimer = 170;
						}
					}
				}
			}
			// Level 3
			else if (stage == 3) {
				// Cross routine
				if (pad.Buttons != oldpad.Buttons) {
					if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
						gotoX = (mouseX-stageX)-cameraX;
						gotoY = (mouseY-stageY)-cameraY;
					}
				}

				// If getting colour from off the screen = freeze. This is meant to prevent that.
				if (stageY+playerY > -cameraY) {
					// Colour specifications
					// black areas
					if (touchColour(RGB(0, 0, 0)))
						dead = 1;
					// red areas
					if (touchColour(RGB(200, 0, 0))) 
						dead = 1;
					// correct switch
					if (touchColour(RGB(252, 201, 77))) 
						doorSwitch = 0;
					// wrong switch
					if (touchColour(RGB(253, 201, 77))) 
						crush = 1;
					// door
					if (touchColour(RGB(73, 53, 4))) 
						dead = 1;
					// finish area
					if (touchColour(RGB(192, 192, 192)))
						transitionLevel();
				}
				else {
					dead = 1;
				}
				
				//The enemies
				enemyTimer--;
				for (i=1; i<=50; i++) {
					if (!(enemy[i].switcher)) {
						if (enemy[i].type == 3) {
							if (enemyTimer < 0) {
								enemy[enemyCur].active = 0;
								if (enemyCur < 7) {
									enemyCur += 2;
									enemy[enemyCur].active = 1;
									enemyTimer = 25;
								}
								else {
									enemyCur = 6+2;
									enemy[i].switcher = 1;
								}
							}
						}
					}
					else if (enemy[i].switcher) {
						if (enemy[i].type == 3) {
							if (enemyTimer < 0) {
								enemy[enemyCur].active = 0;
								if (enemyCur > 2) {
									enemyCur -= 2;
									enemy[enemyCur].active = 1;
									enemyTimer = 25;
								}
								else {
									enemyCur = 1-2;
									enemy[i].switcher = 0;
								}
							}
						}
					}
				}

				// The 2nd obstacle (pulling player in middle)
				if ((playerX > 113) && (playerX < 293) && (playerY > 6) && (playerY < 145)) {
					level3Timer--;
					if (level3Timer < 0) {
						gotoX = 200;
						gotoY = 80;
						level3Timer = 25;
					}
				}

				// The player hits the wrong switch
				if (crush) {
					enemy[8].active = 1; enemy[9].active = 1;
					enemy[8].y++; enemy[9].y++;
				}
			}
			// Level 4
			else if (stage == 4) {
				// Cross routine
				if (pad.Buttons != oldpad.Buttons) {
					if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
						gotoX = (mouseX-stageX)-cameraX;
						gotoY = (mouseY-stageY)-cameraY;
					}
				}

				// If getting colour from off the screen = freeze. This is meant to prevent that.
				if (stageY+playerY > -cameraY) {
					// Colour specifications
					// black areas
					if (touchColour(RGB(0, 0, 0)))
						dead = 1;
					// door
					if (touchColour(RGB(73, 53, 4))) 
						dead = 1;
					// boss
					if (touchColour(RGB(88, 88, 88))) 
						dead = 1;
					// finish area
					if (touchColour(RGB(192, 192, 192)))
						transitionLevel();
				}
				else {
					dead = 1;
				}
				
				// Once player gets past point, switch initiates
				if ((playerY < 215) && (!(doorSwitch))) {
					enemy[1].active = 1;
					doorSwitch = 1;
				}
				
				// This doorSwitch essentially controls the boss too
				if (doorSwitch) {
					if (bossSwitch == 0) {
						// The arm movements
						armTimer--;
						if (armTimer < 0) {
							armXLeftGoto = getRandomNum(0, 25);
							armYLeftGoto = getRandomNum(0, 10);
							armXRightGoto = getRandomNum(0, 25);
							armYRightGoto = getRandomNum(0, 10);
							armTimer = 25;
						}

						if (armXLeftGoto > armXLeft) armXLeft++;
						if (armXLeftGoto < armXLeft) armXLeft--;
						if (armYLeftGoto > armYLeft) armYLeft++;
						if (armYLeftGoto < armYLeft) armYLeft--;
						if (armXRightGoto > armXRight) armXRight++;
						if (armXRightGoto < armXRight) armXRight--;
						if (armYRightGoto > armYRight) armYRight++;
						if (armYRightGoto < armYRight) armYRight--;

						// The boss movements
						if ((enemy[1].x == bossGotoX) && (enemy[1].y == bossGotoY)) {
							enemyTimer--;
							if (enemyTimer < 0) {
								int loll = getRandomNum(0, 9);
								bossGotoX = hotSpots[loll].x;
								bossGotoY = hotSpots[loll].y;
								enemyTimer = 60;
							}
						}
						if (bossGotoX > enemy[1].x) enemy[1].x++;
						if (bossGotoX < enemy[1].x) enemy[1].x--;
						if (bossGotoY > enemy[1].y) enemy[1].y++;
						if (bossGotoY < enemy[1].y) enemy[1].y--;

						// Countdown
						if (countDown < 120)
							bossSwitch++;

						// The countdown till when the boss dies!
						countDownTimer--;
						if (countDownTimer < 0) {
							countDown--;
							countDownTimer = 60;
						}
					}
					else if (bossSwitch == 1) {
						// The arm movements
						armTimer--;
						if (armTimer < 0) {
							armXLeftGoto = 0;
							armYLeftGoto = getRandomNum(0, 30);
							armXRightGoto = 0;
							armYRightGoto = getRandomNum(0, 30);
							armTimer = 20;
						}
						if (armXLeftGoto > armXLeft) armXLeft+=2;
						if (armXLeftGoto < armXLeft) armXLeft-=2;
						if (armYLeftGoto > armYLeft) armYLeft+=2;
						if (armYLeftGoto < armYLeft) armYLeft-=2;
						if (armXRightGoto > armXRight) armXRight+=2;
						if (armXRightGoto < armXRight) armXRight-=2;
						if (armYRightGoto > armYRight) armYRight+=2;
						if (armYRightGoto < armYRight) armYRight-=2;

						// Boss movements
						if (bossGotoX > enemy[1].x) {enemy[1].x+=2.5;}
						else if (bossGotoX < enemy[1].x) {enemy[1].x-=1.5;}
						else if (bossGotoY > enemy[1].y) {enemy[1].y+=2.5;}
						else if (bossGotoY < enemy[1].y) {enemy[1].y-=1.5;}
						else {
							enemyTimer--;
							if (enemyTimer < 0) {
								int loll = getRandomNum(0, 9);
								bossGotoX = hotSpots[loll].x;
								bossGotoY = hotSpots[loll].y;
								backCountTimer = 60;
								enemyTimer = 45;
							}
						}
						// Quick fix for a bug
						backCountTimer--;
						if (backCountTimer < 0) {
							int loll = getRandomNum(0, 9);
							bossGotoX = hotSpots[loll].x;
							bossGotoY = hotSpots[loll].y;
							backCountTimer = 65;
							enemyTimer = 45;
						}

						// Countdown
						if (countDown < 60)
							bossSwitch++;

						// The countdown till when the boss dies!
						countDownTimer--;
						if (countDownTimer < 0) {
							countDown--;
							countDownTimer = 60;
						}
					}
					else if (bossSwitch == 2) {
						// The arm movements
						armTimer--;
						if (armTimer < 0) {
							armXLeftGoto = getRandomNum(0, 30);
							armYLeftGoto = getRandomNum(0, 30);
							armXRightGoto = getRandomNum(0, 30);
							armYRightGoto = getRandomNum(0, 30);
							armTimer = 20;
						}
						if (armXLeftGoto > armXLeft) armXLeft+=2;
						if (armXLeftGoto < armXLeft) armXLeft-=2;
						if (armYLeftGoto > armYLeft) armYLeft+=2;
						if (armYLeftGoto < armYLeft) armYLeft-=2;
						if (armXRightGoto > armXRight) armXRight+=2;
						if (armXRightGoto < armXRight) armXRight-=2;
						if (armYRightGoto > armYRight) armYRight+=2;
						if (armYRightGoto < armYRight) armYRight-=2;

						// Boss movements
						if (bossGotoX > enemy[1].x) {enemy[1].x+=2.5;}
						else if (bossGotoX < enemy[1].x) {enemy[1].x-=1.5;}
						else if (bossGotoY > enemy[1].y) {enemy[1].y+=2.5;}
						else if (bossGotoY < enemy[1].y) {enemy[1].y-=1.5;}
						else {
							enemyTimer--;
							if (enemyTimer < 0) {
								int loll = getRandomNum(0, 9);
								bossGotoX = hotSpots[loll].x;
								bossGotoY = hotSpots[loll].y;
								backCountTimer = 65;
								enemyTimer = 45;
							}
						}
						// Quick fix for a bug
						backCountTimer--;
						if (backCountTimer < 0) {
							int loll = getRandomNum(0, 9);
							bossGotoX = hotSpots[loll].x;
							bossGotoY = hotSpots[loll].y;
							backCountTimer = 60;
							enemyTimer = 45;
						}

						// Countdown
						if (countDown <= 0)
							bossSwitch++;

						// The countdown till when the boss dies!
						countDownTimer--;
						if (countDownTimer < 0) {
							countDown--;
							countDownTimer = 60;
						}
					}
					else if (bossSwitch == 3) {
						enemy[1].active = 0;
						doorSwitch = 0;
					}
				}
			}
			// Level 5
			else if (stage >= 5) {
				if (pad.Buttons != oldpad.Buttons) {
					if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
						playSound(NEXT_PATH);
						initVars();	// Restart everything & go back to menu
					}
				}
			}
			// etc 
		}
		
		
		// If not dead, continue
		if (!(dead)) {
			// Control the 'goto' dot & make sure the player follows it
			if (gotoX >= playerX) playerX++;
			if (gotoX <= playerX) playerX--;
			if (gotoY >= playerY) playerY++;
			if (gotoY <= playerY) playerY--;
		}

		// Update the HUD
		HUD.level = stage;

		// Mouse controller
		// If not dead, continue
		if (!(dead)) {
			// no mouse in the tut stage
			if (stage > 0)
				mouse(sens);
		}
		else {
			// Prompt for start
			if (pad.Buttons & START) {
				playSound(NEXT_PATH);
				initVars();	// Restart everything & go back to menu
			}
		}

		// Check if you're dead or not
		checkDead();

	}
	// Option controls
	else if (curState == STATE_OPTIONS) {
		// Get control
		oldpad = pad;
		sceCtrlReadBufferPositive(&pad, 1);

		if (pad.Buttons != oldpad.Buttons) {
			// Up routine
			if ((pad.Buttons & UP) && (!(oldpad.Buttons & UP))) {
				if (menuOption > 1) menuOption--;
				else menuOption = 4;
				playSound(CLICK_PATH);
			}
			// Down routine
			else if ((pad.Buttons & DOWN) && (!(oldpad.Buttons & DOWN))) {
				if (menuOption < 4) menuOption++;
				else menuOption = 1;
				playSound(CLICK_PATH);
			}

			// Right routine
			if ((pad.Buttons & RIGHT) && (!(oldpad.Buttons & RIGHT))) {
				// Play click sound
				playSound(CLICK_PATH);
				if (menuOption == 1) {
					if (scePowerGetCpuClockFrequencyInt() == 222) scePowerSetClockFrequency(266, 266, 266/2);
					else if (scePowerGetCpuClockFrequencyInt() == 265) scePowerSetClockFrequency(333, 333, 333/2);
					else if (scePowerGetCpuClockFrequencyInt() == 333) scePowerSetClockFrequency(222, 222, 222/2);
				}
				else if (menuOption == 2) state = STATE_OPTIONS;
				else if (menuOption == 3) if (sens < 100) sens++;
			}
			// Left routine
			else if ((pad.Buttons & LEFT) && (!(oldpad.Buttons & LEFT))) {
				// Play click sound
				playSound(CLICK_PATH);
				if (menuOption == 1) {
					if (scePowerGetCpuClockFrequencyInt() == 222) scePowerSetClockFrequency(333, 333, 333/2);
					else if (scePowerGetCpuClockFrequencyInt() == 265) scePowerSetClockFrequency(222, 222, 222/2);
					else if (scePowerGetCpuClockFrequencyInt() == 333) scePowerSetClockFrequency(266, 266, 266/2);
				}
				else if (menuOption == 2) state = STATE_OPTIONS;
				else if (menuOption == 3) if (sens > 1) sens--;
			}

			// Cross routine
			if ((pad.Buttons & CROSS) && (!(oldpad.Buttons & CROSS))) {
				// Play click sound
				playSound(CLICK_PATH);
				if (menuOption == 4) state = STATE_MENU;
			}
		}
	}
}



